@@ -16,6 +16,7 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 import atexit
+import imp
 from optparse import OptionParser
 import os
 from subprocess import PIPE
@@ -32,6 +33,12 @@ parser.disable_interspersed_args()
 parser.add_option('-r', '--report', dest='report',
                   action='store_true',
                   help='Print a detailed coverage report')
+parser.add_option('-l', '--label', dest='label',
+                  action='store_true', default=False,
+                  help='Label result file uniquely')
+parser.add_option('-f', '--file', dest='file',
+                  action='store', default=None,
+                  help='The filename to use as the coverage database')
 parser.add_option('-s', '--summary', dest='summary',
                   action='store_true',
                   help='Print a summary coverage report')
@@ -57,6 +64,25 @@ parser.add_option('-i', '--ignore', dest='ignores',
                   help=('A Python regular expression; modules with names'
                         ' matching this regular expression will be'
                         ' ignored and not have their coverage reported'))
+parser.add_option('--report-literals',
+                  dest='report_conditions_with_literals',
+                  action='store_true', default=False,
+                  help=('Report conditions containing literals as though'
+                        'they were reachable'))
+parser.add_option('--ignore-assertions',
+                  dest='instrument_assertions',
+                  action='store_false', default=True,
+                  help=('Do not instrument and report on the expressions'
+                        ' present in assertions'))
+parser.add_option('--use-metadata-cache',
+                  dest='use_metadata_cache',
+                  action='store_true', default=False,
+                  help=('Use a metadata cache to (possibly) speed up'
+                        ' execution of the target program'))
+parser.add_option('--ignore-comparisons',
+                  dest='instrument_comparisons',
+                  action='store_false', default=True,
+                  help=('Do not instrument comparison expressions'))
 
 def main(argv=None):
     if argv is None:
@@ -64,37 +90,64 @@ def main(argv=None):
     
     opts, args = parser.parse_args(argv)
     
-    if len(args) < 1:
-        parser.print_help()
+    if args and not opts.targets:
+        sys.stdout.write("No targets specified. Use the '-t' option to specify"
+                         " packages to cover\n")
         sys.exit()
     
-    if not opts.targets:
-        sys.stdout.write("No targets specified. Use the '-t' option to specify packages to cover")
+    if opts.label and opts.file:
+        sys.stdout.write("Specify either a coverage database filename (-f) or"
+                         " that the file should be labeled (-l)\n")
         sys.exit()
     
-    coverage = Coverage()
-    coverage.start(opts.targets, opts.ignores)
-    
     xml_filename = os.path.abspath('instrumental.xml')
     
-    sourcefile = args[0]
-    environment = {'__name__': '__main__',
-                   '__file__': sourcefile,
-                   }
-    sys.argv = args[:]
+    cwd = os.getcwd()
+    
+    coverage = Coverage(opts, cwd)
+    old_main = sys.modules['__main__']
     try:
-        here = os.getcwd()
-        execfile(sourcefile, environment)
+        if args:
+            coverage.start(opts.targets, opts.ignores)
+            
+            
+            sourcefile = args[0]
+            # our initial attempt at providing an execution environment under
+            # which the target script can run was a flop. We need to create
+            # a false __main__ module or we'll run into problems like those
+            # reported in Bitbucket Issue #9.
+            
+            # environment = {'__name__': '__main__',
+            #                '__file__': sourcefile,
+            #                }
+            
+            # This stanza is taken mostly from coverage.py:coverage/execfile.py
+            new_main = imp.new_module('__main__')
+            sys.modules['__main__'] = new_main
+            new_main.__file__ = sourcefile
+            if sys.version_info[0] < 3:
+                new_main.__builtins__ = sys.modules['__builtin__']
+            else:
+                new_main.__builtins__ = sys.modules['builtin']
+            environment = new_main.__dict__
+            
+            sys.argv = args[:]
+            execfile(sourcefile, environment)
     finally:
+        sys.modules['__main__'] = old_main
+        
+        os.chdir(cwd)
+        if coverage.started:
+            coverage.stop()
+            coverage.save()
         if any([opts.summary,
                 opts.report,
                 opts.statements,
                 opts.xml,
                 opts.html]):
-            coverage.stop()
             sys.stdout.write("\n")
-            recorder = coverage.recorder
-            report = ExecutionReport(here, recorder.constructs, recorder.statements, recorder.sources)
+            recorder = coverage.load()
+            report = ExecutionReport(cwd, recorder.metadata, opts)
             if opts.summary:
                 sys.stdout.write(report.summary() + "\n")
             if opts.report:
