@@ -4,6 +4,7 @@ from instrumental.constructs import LogicalAnd
 from instrumental.constructs import LogicalBoolean
 from instrumental.constructs import LogicalOr
 from instrumental.recorder import ExecutionRecorder
+from instrumental.test import DummyConfig
 
 class ThreePinTestCase(object):
     
@@ -15,6 +16,7 @@ class ThreePinTestCase(object):
                                        ],
                                lineno=6,
                                col_offset=4)
+        self.label = '6.1'
 
 class TwoPinTestCase(object):
     
@@ -25,6 +27,7 @@ class TwoPinTestCase(object):
                                        ],
                                lineno=6,
                                col_offset=4)
+        self.label = '6.1'
 
 class TestLogicalAnd(ThreePinTestCase):
     
@@ -33,10 +36,10 @@ class TestLogicalAnd(ThreePinTestCase):
         self.node.op = ast.And()
     
     def _makeOne(self):
-        return LogicalAnd(self.modulename, self.node, None)
+        return LogicalAnd(self.modulename, self.label, self.node, set())
     
     def test_constructor(self):
-        assert LogicalAnd(self.modulename, self.node, None)
+        assert LogicalAnd(self.modulename, self.label, self.node, set())
     
     def test_has_conditions(self):
         assert hasattr(self._makeOne(), 'conditions')
@@ -67,41 +70,41 @@ class TestLogicalAnd(ThreePinTestCase):
         assert "T T F" == and_.description(3), and_.description(3)
     
     def _expect_result(self, *set_conditions):
-        expected_result = "\n".join(["LogicalAnd -> somename:6 < (a and b and c) >",
+        expected_result = "\n".join(["LogicalAnd -> somename:6.1 < (a and b and c) >",
                                      "",
-                                     "T T T ==> %s" % ("T T T" in set_conditions),
-                                     "F * * ==> %s" % ("F * *" in set_conditions),
-                                     "T F * ==> %s" % ("T F *" in set_conditions),
-                                     "T T F ==> %s" % ("T T F" in set_conditions),
+                                     "T T T ==> %s" % ('X' if "T T T" in set_conditions else ''),
+                                     "F * * ==> %s" % ('X' if "F * *" in set_conditions else ''),
+                                     "T F * ==> %s" % ('X' if "T F *" in set_conditions else ''),
+                                     "T T F ==> %s" % ('X' if "T T F" in set_conditions else ''),
                                      ])
         return expected_result
                                      
     def test_all_true(self):
         and_ = self._makeOne()
-        and_.record(True, 0)
-        and_.record(True, 1)
-        and_.record(True, 2)
+        and_.record(True, 0, 'X')
+        and_.record(True, 1, 'X')
+        and_.record(True, 2, 'X')
         expected_result = self._expect_result('T T T')
         assert expected_result == and_.result()
         
     def test_F_T_T(self):
         and_ = self._makeOne()
-        and_.record(False, 0)
+        and_.record(False, 0, 'X')
         expected_result = self._expect_result('F * *')
         assert expected_result == and_.result(), and_.result()
         
     def test_T_F_T(self):
         and_ = self._makeOne()
-        and_.record(True, 0)
-        and_.record(False, 1)
+        and_.record(True, 0, 'X')
+        and_.record(False, 1, 'X')
         expected_result = self._expect_result('T F *')
         assert expected_result == and_.result()
         
     def test_T_T_F(self):
         and_ = self._makeOne()
-        and_.record(True, 0)
-        and_.record(True, 1)
-        and_.record(False, 2)
+        and_.record(True, 0, 'X')
+        and_.record(True, 1, 'X')
+        and_.record(False, 2, 'X')
         expected_result = self._expect_result('T T F')
         assert expected_result == and_.result()
         
@@ -112,7 +115,7 @@ class TestLogicalAnd2Pin(TwoPinTestCase):
         self.node.op = ast.And()
     
     def _makeOne(self):
-        return LogicalAnd(self.modulename, self.node, None)
+        return LogicalAnd(self.modulename, self.label, self.node, set())
     
     def test_2_pin_and_condition_0(self):
         and_ = self._makeOne()
@@ -137,7 +140,7 @@ class TestLogicalOr(ThreePinTestCase):
         self.node.op = ast.Or()
     
     def _makeOne(self):
-        return LogicalOr(self.modulename, self.node, None)
+        return LogicalOr(self.modulename, self.label, self.node, set())
     
     def test_has_conditions(self):
         or_ = self._makeOne()
@@ -173,41 +176,41 @@ class TestLogicalOr(ThreePinTestCase):
         assert "Other" == or_.description(4)
     
     def _expect_result(self, *set_conditions):
-        expected_result = "\n".join(["LogicalOr -> somename:6 < (a or b or c) >",
+        expected_result = "\n".join(["LogicalOr -> somename:6.1 < (a or b or c) >",
                                      "",
-                                     "T * * ==> %s" % ("T * *" in set_conditions),
-                                     "F T * ==> %s" % ("F T *" in set_conditions),
-                                     "F F T ==> %s" % ("F F T" in set_conditions),
-                                     "F F F ==> %s" % ("F F F" in set_conditions),
+                                     "T * * ==> %s" % ('X' if "T * *" in set_conditions else ''),
+                                     "F T * ==> %s" % ('X' if "F T *" in set_conditions else ''),
+                                     "F F T ==> %s" % ('X' if "F F T" in set_conditions else ''),
+                                     "F F F ==> %s" % ('X' if "F F F" in set_conditions else ''),
                                      ])
         return expected_result
                                      
     def test_T_F_F(self):
         or_ = self._makeOne()
-        or_.record(True, 0)
+        or_.record(True, 0, 'X')
         expected_result = self._expect_result('T * *')
         assert expected_result == or_.result(), or_.result()
         
     def test_F_T_F(self):
         or_ = self._makeOne()
-        or_.record(False, 0)
-        or_.record(True, 1)
+        or_.record(False, 0, 'X')
+        or_.record(True, 1, 'X')
         expected_result = self._expect_result('F T *')
         assert expected_result == or_.result()
         
     def test_F_F_T(self):
         or_ = self._makeOne()
-        or_.record(False, 0)
-        or_.record(False, 1)
-        or_.record(True, 2)
+        or_.record(False, 0, 'X')
+        or_.record(False, 1, 'X')
+        or_.record(True, 2, 'X')
         expected_result = self._expect_result('F F T')
         assert expected_result == or_.result()
         
     def test_F_F_F(self):
         or_ = self._makeOne()
-        or_.record(False, 0)
-        or_.record(False, 1)
-        or_.record(False, 2)
+        or_.record(False, 0, 'X')
+        or_.record(False, 1, 'X')
+        or_.record(False, 2, 'X')
         expected_result = self._expect_result('F F F')
         assert expected_result == or_.result()
         
@@ -218,7 +221,7 @@ class TestLogicalOr2Pin(TwoPinTestCase):
         self.node.op = ast.Or()
     
     def _makeOne(self):
-        return LogicalOr(self.modulename, self.node, None)
+        return LogicalOr(self.modulename, self.label, self.node, set())
     
     def test_2_pin_or_condition_0(self):
         or_ = self._makeOne()
@@ -246,9 +249,10 @@ class TestLogicalBoolean(object):
                                lineno=17,
                                col_offset=24,
                                )
+        self.label = '17.1'
     
     def _makeOne(self):
-        return LogicalAnd(self.modulename, self.node, None)
+        return LogicalAnd(self.modulename, self.label, self.node, set())
     
     def test_modulename(self):
         construct = self._makeOne()
@@ -268,14 +272,14 @@ class TestLogicalBoolean(object):
     
     def test_conditions(self):
         construct = self._makeOne()
-        assert {0: False,
-                1: False,
-                2: False,
+        assert {0: set(),
+                1: set(),
+                2: set(),
                 } == construct.conditions, construct.conditions
     
     def test_number_of_conditions(self):
         construct = self._makeOne()
-        assert 3 == construct.number_of_conditions()
+        assert 3 == construct.number_of_conditions(False)
     
     def test_number_of_conditions_hit(self):
         construct = self._makeOne()
@@ -283,15 +287,15 @@ class TestLogicalBoolean(object):
     
     def test_conditions_missed(self):
         construct = self._makeOne()
-        assert 3 == construct.conditions_missed()
+        assert 3 == len(construct.conditions_missed(False))
     
     def test_initial_result(self):
         construct = self._makeOne()
-        expected_result = "\n".join(["LogicalAnd -> somename.subname:17 < (a and b) >",
+        expected_result = "\n".join(["LogicalAnd -> somename.subname:17.1 < (a and b) >",
                                      "",
-                                     "T T ==> False",
-                                     "F * ==> False",
-                                     "T F ==> False",
+                                     "T T ==> ",
+                                     "F * ==> ",
+                                     "T F ==> ",
                                      ])
         assert expected_result == construct.result()
 
@@ -299,17 +303,41 @@ class TestLiteralInConstruct(object):
     
     def setup(self):
         # Reset recorder
-        ExecutionRecorder.reset()
         self.modulename = 'somename'
-        self.node = ast.BoolOp(op=ast.Or(),
-                               values=[ast.Name(id='a'),
-                                       ast.Str(s='None'),
-                                       ],
-                               lineno=6,
-                               col_offset=4)
+        self.source = """
+
+
+
+
+x = a or None"""
+        self.label = '6.2'
     
+    def _makeOne(self):
+        from instrumental.metadata import MetadataGatheringVisitor
+        metadata = MetadataGatheringVisitor.analyze(DummyConfig(),
+                                                    self.modulename,
+                                                    self.source, 
+                                                    {6: set()})
+        return metadata.constructs[self.label]
+        
     def test_presence_of_a_literal(self):
-        recorder = ExecutionRecorder.get()
-        recorder.add_BoolOp(self.modulename, self.node, [], None)
-        construct = list(recorder.constructs.values())[0]
+        construct = self._makeOne()
         assert "literal" in construct.result(), construct.result()
+    
+    def test_unreachable_condition(self):
+        from instrumental.constructs import UnreachableCondition
+        construct = self._makeOne()
+        assert 2 == construct.number_of_conditions(False)
+        assert not construct.conditions[0]
+        assert construct.conditions[1] == set([UnreachableCondition])
+        assert not construct.conditions[2]
+        assert set(['T *', 'F F']) == set(construct.conditions_missed(False))
+    
+    def test_count_literals(self):
+        from instrumental.constructs import UnreachableCondition
+        construct = self._makeOne()
+        assert 3 == construct.number_of_conditions(True)
+        assert not construct.conditions[0]
+        assert construct.conditions[1] == set([UnreachableCondition])
+        assert not construct.conditions[2]
+        assert set(['T *', 'F T', 'F F']) == set(construct.conditions_missed(True))
